# Laravel Squire Client

## Install

### Add Myriad Repository

The client is distributed internally with Satis. First add the repository to `composer.json`:

```json
  "repositories": [
    {
      "type": "composer",
      "url": "https://satis.myriadmobile.com"
    }
  ]
```

### Require the Dependency

Require this package with composer using the following command:

```bash
composer require myriad/squire-client
```

### Install Service Provider

After updating composer, add the service provider to the `providers` array in `config/app.php`:

```php
Myriad\Squire\SquireServiceProvider::class,
```

### Install Facade (Optional)

If you choose, register the facade in the `aliases` array in `config/app.php`:

```php
'Squire' => Myriad\Squire\Facades\Squire::class,
```

### Configuration

By default, the following environment variables are read to setup Squire:

* `SQUIRE_TOKEN` => The application's unique server token
* `SQUIRE_SERVER` => The url to the Squire server. Default: https://squire.myriadapps.com

If environment variables are not sufficient, the configuration file can be published using:

```bash
php artisan vendor:publish --provider=myriad/squire-client --tag=config
```

## Usage

Note. Examples make use of the Facade. For best practice, inject an instance of `Myriad\Squire\Contracts\Squire`.

### URLs

#### Files

To get the url for file, use the file builder:

```php
$url = Squire::file($key)->build();
```

`$key` is an identifier assigned to a file, most often this is a filename. e.g. `myfile.pdf`. Keys can also contain
 slashes to help organize your files in to folders: `myfolder/myfile.pdf`.

#### Images

To get the url for an image, use the image builder:

```php
$url = Squire::image($key)->with(200)->grayscale()->build();
```

__NOTE: To be processed correctly, the image key must end with a known image file extension.__

### Get File

Squire makes retrieving your files painless:

```php
$resource = Squire::get($key);
```

The file will be received as a resource, to get the contents of the file use:

```php
$string = stream_get_contents($resource);
```

or put the contents to a file:

```php
$string = file_put_contents('myfile.pdf');
```

### Get Image

Images can be received just like any other file, however it is also possible to receive a manipulated image:

```php
$key = Squire::image($key)->with(200)->grayscale()->build(true);
$resource = Squire::get($key);
```

### Put File

Pushing a file to Squire is as easy as:

```php
Squire::put($resource, $key);
```

`$resource` can be a binary string, file, and nearly anything else. It works directly with Laravel's UploadedFile to
make uploads as simple as possible.

__NOTE: It is possible to replace a source file, however, files already in the cache will not reflect the changes.__

### Delete File

Deleting a file is simple:

```php
Squire::delete($key);
```

__NOTE: Currently only source files are deleted, this means cached files will continue to be served until they expire.__