<?php namespace Myriad\Squire\Contracts;

use GuzzleHttp\Exception\ServerException;
use Myriad\Squire\SquireConfiguration;
use Myriad\Squire\SquireFile;
use Myriad\Squire\SquireImage;

interface Squire {

  /**
   * Create a new instance of squire
   *
   * @param \Myriad\Squire\SquireConfiguration $config
   */
  public function __construct(SquireConfiguration $config);

  /**
   * Get a file from Squire
   *
   * @param string $key the file key
   *
   * @return resource
   * @throws ServerException
   */
  public function get($key);

  /**
   * Put file to Squire
   *
   * @param resource|string|\SplFileInfo $resource
   * @param string $key the file key
   *
   * @throws ServerException
   */
  public function put($resource, $key);

  /**
   * Delete file from Squire
   *
   * @param string $key the file key
   *
   * @throws ServerException
   */
  public function delete($key);

  /**
   * Make a SquireImage
   *
   * @param string $key the file key
   *
   * @return SquireImage
   */
  public function image($key);

  /**
   * Make a SquireFile
   *
   * @param string $key the file key
   *
   * @return SquireFile
   */
  public function file($key);

  /**
   * Get the Squire configuration
   *
   * @return SquireConfiguration
   */
  public function getConfiguration();

}