<?php namespace Myriad\Squire;

use Myriad\Squire\Contracts\Squire;

class SquireClient implements Squire {

  /**
   * @inheritdoc
   */
  public function __construct(SquireConfiguration $config) {
    $this->config = $config;
  }

  /**
   * @inheritdoc
   */
  public function get($key) {
    return $this->doRequest('GET', $key)->getBody()->detach();
  }

  /**
   * @inheritdoc
   */
  public function put($resource, $key) {
    $body = $this->getRequestBody($resource);
    $this->doRequest('PUT', $key, [
      'body' => $body
    ]);
    if (is_resource($body)) {
      @fclose($body);
    }
  }

  /**
   * @inheritdoc
   */
  public function delete($key) {
    $this->doRequest('DELETE', $key);
  }

  /**
   * Creates and sends an authorized request
   *
   * @param string $method the http method
   * @param string $key the file key
   * @param array $options additional guzzle options
   *
   * @return \GuzzleHttp\Message\FutureResponse|\GuzzleHttp\Message\ResponseInterface|\GuzzleHttp\Ring\Future\FutureInterface|null
   */
  private function doRequest($method, $key, $options = []) {
    $defaultOptions = [
      'headers' => [
        'Authorization' => $this->getConfiguration()->getServerToken()
      ]
    ];
    $client = new \GuzzleHttp\Client();
    $request = $client->createRequest($method, $this->config->getServerUrl() . '/api/v1/files/'
                                               . $key, array_merge($defaultOptions, $options));

    return $client->send($request);
  }

  /**
   * Open a handle to the resource
   * @param resource|string|\SplFileInfo $resource
   *
   * @return resource|string
   */
  private function getRequestBody($resource) {
    // if we already have a file handle, rewind and return
    if (is_resource($resource)) {
      @rewind($resource);
      return $resource;
    }

    // if the resource extends SplFileInfo, open a handle to the real path
    if ($resource instanceof \SplFileInfo) {
      return fopen($resource->getRealPath(), 'r');
    }

    // if the resource is a readable file path, open a handle to it
    if (is_readable($resource)) {
      return fopen($resource, 'r');
    }

    // otherwise just return the resource as is
    return $resource;
  }

  /**
   * @inheritdoc
   */
  public function image($key) {
    return new SquireImage($key, $this->getConfiguration());
  }

  /**
   * @inheritdoc
   */
  public function file($key) {
    return new SquireFile($key, $this->getConfiguration());
  }

  /**
   * @inheritdoc
   */
  public function getConfiguration() {
    return $this->config;
  }

}