<?php namespace Myriad\Squire;

class SquireFile {

  /**
   * @var string
   */
  private $key;

  /**
   * @var array
   */
  private $params = [];

  /**
   * @var \Myriad\Squire\SquireConfiguration
   */
  private $configuration;

  /**
   * Create a new SquireFile builder
   *
   * @param string $key
   * @param SquireConfiguration $configuration
   */
  public function __construct($key, SquireConfiguration $configuration) {
    $this->key = $key;
    $this->configuration = $configuration;
  }

  /**
   * @param string $name the argument name
   * @param mixed $value the argument value
   *
   * @return $this
   */
  public function param($name, $value) {
    $this->params[$name] = (string)$value;
    return $this;
  }

  /**
   * Magic method for parameter chaining
   *
   * @param string $name
   * @param array $arguments
   *
   * @return $this
   */
  public function __call($name, $arguments) {
    return $this->param($name, implode(',', $arguments));
  }

  /**
   * Build the url string
   *
   * @param bool|false $relative
   *
   * @return string the url string for the image
   */
  public function build($relative = false) {
    $build = trim($this->key, '/');
    if (!empty($this->params)) {
      ksort($this->params);
      $build .= '?' . http_build_query($this->params);
    }
    if (!$relative) {
      $domain = rtrim($this->configuration->getDomain(), '/');
      $prefix = trim($this->configuration->getPrefix(), '/');
      $build = 'https://' . $domain . '/' . $prefix . '/' . $build;
    }

    return $build;
  }

  /**
   * Build the url when displaying as a string
   *
   * @return string
   */
  public function __toString() {
    return $this->build();
  }

}