<?php namespace Myriad\Squire;

use Illuminate\Support\ServiceProvider;

class SquireServiceProvider extends ServiceProvider {

  /**
   * Indicates if loading of the provider is deferred.
   *
   * @var bool
   */
  protected $defer = true;

  /**
   * Perform post-registration booting of services.
   *
   * @return void
   */
  public function boot() {
    $this->publishes([__DIR__ . '/../config/squire.php' => config_path('squire.php')]);
  }

  /**
   * Register the service provider.
   *
   * @return void
   */
  public function register() {
    $this->mergeConfigFrom(__DIR__ . '/../config/squire.php', 'squire');

    $this->registerSquire();

    $this->app->alias('squire', 'Myriad\Squire\Contracts\Squire');
    $this->app->alias('squire', 'Myriad\Squire\SquireClient');
  }

  /**
   * Register the Squire instance
   */
  public function registerSquire() {
    $this->app->singleton('squire', function ($app) {
      $config = new SquireConfiguration(config('squire'), $app['cache.store']);

      return new SquireClient($config);
    });
  }

  /**
   * Get the services provided by the provider.
   *
   * @return array
   */
  public function provides() {
    return ['squire', 'Myriad\Squire\Contracts\Squire', 'Myriad\Squire\SquireClient'];
  }

}